<?php

    require_once('./libs/ajaxac/AjaxACApplication.class.php');

    class Suggest extends AjaxACApplication
    {

        /**
         * Time to wait after a keypress before fetching suggestions
         */
        var $keypress_timeout_ms = 350;

        /**
         * Maximum number of suggestions to return
         */
        var $suggestion_limit = 15;

    	/**
    	 * Constructor. Creates the necessary action and then inits widgets
    	 *
    	 */
    	public function __construct()
    	{
			parent::AjaxACApplication();
			$this->registerActions('getsuggestions');
$this->registerActions('getsuggestions2');
			$this->setup();
    	}


        /**
         * Create necessary widgets and events
         */
        function setup()
        {
            // utility functions for the app
            $this->addJsLib('suggest.js');

            // the form input where users type their query
            $query = $this->createWidget('query');
            $query->addEvent(AJAXAC_EV_ONKEYDOWN, 'querykeydown');
            $this->addWidget($query);


            // the div where results will be displayed
            $results = $this->createWidget('results');
            $results->addEvent(AJAXAC_EV_ONLOAD, 'resultsload');
            $this->addWidget($results);


            // the countdown timer that is started/reset after a key is pressed
            require_once('./libs/ajaxac/Widgets/AjaxACWidgetTimer.class.php');
            $timer = new AjaxACWidgetTimer($this, 'gsctimer');
            $timer->addEvent(AJAXAC_EV_ONTIMEREXPIRE, 'timerexpire');
            $timer->setTimeoutFromInt($this->keypress_timeout_ms);
            $this->addWidget($timer);
        }

        /**
         * Handle the getsuggestion HTTP subrequest
         */
        function action_getsuggestions()
        {
            $suggestions = array();
	     $this->getSuggestions2($this->getRequestValue('keywords'), $suggestions);
	     $this->getSuggestions($this->getRequestValue('keywords'), $suggestions);
	     $this->sendResponseData('jsarray', $suggestions);            
        }

        /**
         * Initialise results output by clearing and hiding it
         */
        function event_resultsload(&$widget, $event)
        {
            return "function() { gsc_emptyresults(this); }";
        }

        /**
         * Handle various keypresses that occur on query input
         */
        function event_querykeydown(&$widget, $event)
        {
            $callback = "
                            function(e)
                            {
                                key = ajaxac_getkeycode(e);
                                switch (key) {
                                    case 27: // escape
                                        gsc_hide(%1\$s);
                                        return false;
                                        break;
                                    case 38: // up arrow
                                        gsc_handleup(%1\$s, %2\$s);
                                        return false;
                                        break;
                                    case 40: // down arrow
                                        gsc_handledown(%1\$s, %2\$s);
                                        return false;
                                        break;
                                    default:
                                        %3\$s.start();
                                }
                                return true;
                            }
                        ";

            $callback = sprintf($callback,
                                $this->getHookName('results'),
                                $this->getHookName('query'),
                                $this->getHookName('gsctimer'));

            return $callback;
        }

        /**
         * Handle the time expiring (i.e. initiate the HTTP sub request
         */
        function event_timerexpire(&$widget, $event)
        {
            require_once('./libs/ajaxac/Widgets/AjaxACWidgetXMLHttpRequest.class.php');
            $xmlhttp = new AjaxACWidgetXMLHttpRequest($this, 'gscfetch', AJAXAC_METH_GET);
            $xmlhttp->setFilenameFromString($this->getApplicationUrl('getsuggestions'));
            $xmlhttp->addParamFromHookValue('keywords', '_keywords', '');
            $xmlhttp->addEvent(AJAXAC_EV_ONXMLHTTPSUCCESS, 'handlesuggestions');
            $this->addWidget($xmlhttp);

            $callback = "
                            function()
                            {
                                _keywords = gsc_getquery(%1\$s, %2\$s.value);
                                if (_keywords.length == 0)
                                    return false;
                                try {
                                    %3\$s
                                }
                                catch (e) { }


                                return false;
                            }
                        ";

            $callback = sprintf($callback,
                                $this->getHookName('results'),
                                $this->getHookName('query'),
                                $xmlhttp->getJsCode());

            return $callback;
        }

        /**
         * Receive the suggestions and load them into results box
         */
        function event_handlesuggestions(&$widget, $event)
        {
            $callback = "
                            function()
                            {
                                _data = ajaxac_receivejsarray(%1\$s.responseText);
                                gsc_emptyresults(%2\$s);
                                if (_data.length > 0) {
                                    for (i = 0; i < _data.length; i++) {
                                        gsc_addresult(%2\$s, %3\$s, _data[i][0], _data[i][1], i == 0);
                                    }
                                    gsc_show(%2\$s);
                                }
                            }
                        ";
            $callback = sprintf($callback,
                                $widget->getHookName(),
                                $this->getHookName('results'),
                                $this->getHookName('query'));
            return $callback;
        }

        /**
         * Get the suggestions from the database and write to array
         */
        function getSuggestions($prefix, &$arr)
        {
        	$db = db_connect();
        	// firstly clean up the data
        	$prefix = ltrim(preg_replace('/[^a-z0-9_. ]/', '', strtolower($prefix)));
        	$prefix = preg_replace('/\s+/', ' ', $prefix);
        	if (strlen($prefix) > 0)
        	{
        		$prefix = mysql_real_escape_string($prefix);
        		$res = $db->query("
        			SELECT CONCAT_WS(' ',surname,middlename,name)
					FROM `author`
	                WHERE name LIKE '$prefix%' OR surname LIKE '$prefix%' 
					ORDER BY surname
					LIMIT $this->suggestion_limit");

        		while ($row = $res->fetchRow(DB_FETCHMODE_ORDERED))
        		{
        			$arr[] = array($row[0], 0);
        		}
        	}
        }

	function getSuggestions2($prefix, &$arr)
        {
        	$db = db_connect();
        	// firstly clean up the data
        	$prefix = ltrim(preg_replace('/[^a-z0-9_. ]/', '', strtolower($prefix)));
        	$prefix = preg_replace('/\s+/', ' ', $prefix);
        	if (strlen($prefix) > 0)
        	{
        		$prefix = mysql_real_escape_string($prefix);
        		$res = $db->query("
        			SELECT CONCAT_WS(' ',title)
					FROM `publication`
	                WHERE title LIKE '$prefix%' 
					ORDER BY title
					LIMIT $this->suggestion_limit");

        		while ($row = $res->fetchRow(DB_FETCHMODE_ORDERED))
        		{
        			$arr[] = array($row[0], 0);
        		}
        	}
        }


    }
?>