<?php


/**
 * Document provider.
 *
 */
class DocumentProvider
{
	private $db;
	private $auth_module;
	private $path_prefix;

	/**
	 * Constructor
	 *
	 * @param PEAR::DB $db
	 * @param array $permitted_extensions permitted document extensions
	 * @param bool $called_from_admin optional flag
	 */
	public function __construct(&$db, $called_from_admin=false)
	{
		$this->db = $db;
		$this->auth_module = new AuthModule($this->db);
		if (!is_numeric($this->auth_module->get_id()))
			throw new Exception('You are not permitted to operate over the documents. Log In First.');
		if ($called_from_admin)
			$this->path_prefix = '..';
		else
			$this->path_prefix = '.';
	}

	/**
	 * According to given parameters, read requested document and send it to the peer.
	 *
	 * @param int $publication_id
	 * @param string $filename
	 */
	public function send_file($publication_id, $filename)
	{
		$fsp = explode('.', $filename);
		$extension = $fsp[count($fsp) - 1];
		if (!in_array($extension, array('pdf', 'ps', 'ppt', 'txt', 'doc', 'odt', 'zip', 'rar'))) //avoid downloading scripts etc.
			throw new Exception("`$extension` is not known document extension.");

		switch($extension)
		{
			case "pdf": $ctype="application/pdf"; break;
			case "ps": $ctype="application/ps"; break;
			case "doc": $ctype="application/msword"; break;
			case "odt": $ctype="application/odt"; break;
			case "txt": $ctype="text/plain"; break;
			case "zip": $ctype="application/zip"; break;
			case "rar": $ctype="application/rar"; break;
			case "ppt": $ctype="application/vnd.ms-powerpoint"; break;
			default: $ctype="application/force-download";
		}
		$path = "$this->path_prefix/storage/$publication_id/$filename";
		header('Last-Modified: '.gmdate('D, d M Y H:i:s', filemtime($fn)).' GMT', true, 200);
		header('Content-Length: '.filesize($path));
		header('Content-Type: '.$ctype);
		header("Content-Transfer-Encoding: binary");
		header("Content-Disposition: attachment; filename=".$filename.";");
		print file_get_contents($path);
	}

	/**
	 * Returns html links to the files associated with the publication
	 *
	 * @param int $publication_id
	 * @return string html links
	 */
	public function get_file_links($publication_id)
	{
		$path="$this->path_prefix/storage/$publication_id";
		if(file_exists($path)) {
			$handle=opendir($path);
			while ($file = readdir($handle)) {
				if (is_file($path.'/'.$file)){
					$filenames[]=$file;
				}
			}
		}
		$res = '';
		if (is_array($filenames)) foreach ($filenames as $filename)
		{
			if (ends_with($filename, 'pdf')) {
				$icon = "$this->path_prefix/graphics/pdf_icon.gif";
			} elseif (ends_with($filename, 'ps')) {
				$icon = "$this->path_prefix/graphics/ps_icon.gif";
			} elseif (ends_with($filename, 'doc')) {
				$icon = "$this->path_prefix/graphics/doc_icon.gif";
			}
			$imglink = "<img border=\"0\" src=\"$icon\" alt=\"document icon\" />";
			$res .= '<a href="'.$this->path_prefix.'/document_provider.php?pid='.$publication_id.'&amp;f='.$filename.'">'.$imglink.' '.$filename.'</a><br />';
		}
		return $res;
	}


}
?>