<?php
//require_once('Contents_waiting_checked.inc.php');
require_once('Contents_publications.inc.php');
require_once('Contents_attributes.inc.php');
require_once('Contents_categories.inc.php');
require_once('Contents_authors.inc.php');
require_once('Contents_publishers.inc.php');
require_once('Contents_journals.inc.php');
require_once('Contents_conferences.inc.php');
require_once('Contents_templates.inc.php');
require_once('Contents_users.inc.php');


/**
 * declares contents interface basic class
 *
 */
abstract class Contents
{
	protected $post;
	protected $get;
	protected $admin_uid;
	protected $db;
	public $smarty;
	protected $auth_module;
	protected $strict_mode;

	/*	Uses sessions:
		$_SESSION['feedback_modify'] - saved form data of admin_tab12.tpl template of section modify
		$_SESSION['tmp_attr'] - new extra attributes temporarily added by admin
		$_SESSION['attributes_deleted'] - extra ettributes flag
		$_SESSION['query_string'] - last query string for purposes of tab 1 - modify publication
		$_SESSION['selected_authors'] - authors choosen in  tab 1 - modify publication. This array is automatically deleted when clicking to another tab
	*/

	/**
	 * Factory method - takes number of the tab and creates appropriate successor class
	 * containing logic of a particular tab page.
	 *
	 * @param integer $tab_no number of the tab (1 - 11)
	 */
	public static function create($tab_no)
	{
		$post = clean_rec($_POST);
		$get = clean_rec($_GET);
		switch($tab_no) {
			//publications waiting for check
			case('1'): $contents = new Contents_publications($post, $get, 1, true); break;
			//publications all
			case('2'): $contents = new Contents_publications($post, $get, 2, true); break;
			//publications all
			case('3'): $contents = new Contents_publications($post, $get, 3, true); break;
			//attributes
			case('4'): $contents = new Contents_attributes($post, $get); break;
			//categories
			case('5'): $contents = new Contents_categories($post, $get); break;
			//authors
			case('6'): $contents = new Contents_authors($post, $get); break;
			//publishers
			case('7'): $contents = new Contents_publishers($post, $get); break;
			//journals
			case('8'): $contents = new Contents_journals($post, $get); break;
			//conferences
			case('9'): $contents = new Contents_conferences($post, $get); break;
			//templates
			case('10'): $contents = new Contents_templates($post, $get); break;
			//users
			case('11'): $contents = new Contents_users($post, $get); break;
			//my publications
			case('12'): $contents = new Contents_publications($post, $get, 12, true); break;

		}
		return $contents ;
	}

	/**
	 * Constructor
	 *
	 * @param string $post  $_POST input
	 * @param string $get  $_GET input
	 * @param boolean $needs_admin_rights
	 */
	public function __construct($post, $get, $needs_admin_rights=true)
	{
		
		$this->post = $post;
		$this->get = $get;
		$this->db = db_connect();
		$this->auth_module = new AuthModule($this->db);

		$this->smarty = new Smarty();
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;

		global $account_url;
		$this->smarty->assign('account_url', $account_url);
		global $app_path;
		$this->smarty->assign('app_path', $app_path);
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		//if no user logged in, force authorization
		if (!is_numeric($this->auth_module->get_id()))
		{
			$this->smarty->assign('refresh', "1 ; URL=../account/index.php?ref=/");
			$this->smarty->assign('operation_error', 'You must log in.');
			$this->smarty->display('admin_login.tpl');
			die();
		}


		$this->smarty->assign('level', $this->auth_module->get_level());
		
	}

	

	/**
	 * Execute contents 
	 *
	 */
	abstract public function execute();

}

?>