<?php
require_once('Contents.inc.php');

/**
 * Managing journals
 *
 */
final class Contents_journals extends Contents
{

	/**
	 * Execute contents tab 8
	 *
	 */
	public function execute()
	{
		include '../help/Help.php';
		if ($this->post['add_journal']) {
			try {
				add_journal($this->db, $this->post['name'], $this->post['issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been added');
				$this->display_journals_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_journals_list();
			}
		} elseif ($ea=$this->post['edit_journal']) {//$ea is array with only one element in format: $journal_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $journal_id => $v) {
					$this->display_edit_journal($journal_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_journal']) {
			try {
				update_journal($this->db, $this->post['journal_id'], $this->post['name'], $this->post['issn'], $this->auth_module->get_id(), true);
				$this->smarty->assign('operation_success', 'Journal has been updated');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=8');
				$this->smarty->display('contents_journals.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_journal($this->post['journal_id']);
			}
		} elseif ($da=$this->post['delete_journal']) {//$da is array with only one element in format: [...] => $journal_id
			try {
				if(is_array($da)) foreach($da as $journal_id => $v) {
					$this->delete_journal($journal_id);
					$this->smarty->assign('operation_success', 'Journal has been deleted.');
					$this->display_journals_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		}
		elseif ($this->post['delete_files'] && $this->get['pid']){
			try {
				$this->delete_files($this->get['pid']);
				$this->smarty->assign('operation_success', 'Files has been deleted.');
				$this->display_overview($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_overview($this->get['pid']);
			}
		} elseif ($this->post['add_file'] && $this->get['pid']) {
			try {
				$this->add_file($this->get['pid']);
				$this->smarty->assign('operation_success', 'File has been added.');
				$this->display_overview($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_overview($this->get['pid']);
			}
		}
		 elseif ($ea=$this->post['display_publications']) {
			try {
				if(is_array($ea)) foreach($ea as $journal_id => $v) {
					$_SESSION["journal_id"] = $journal_id;

					$this->display_publications($journal_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($ea=$this->post['overview']) {
			try {
				if(is_array($ea)) foreach($ea as $publication_id => $v) {
					$this->display_overview($publication_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} else {
			$this->display_journals_list();
		}
	}

	/**
	 * Show list of journals
	 *
	 */
	private function display_journals_list()
	{
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY issn"; break;
			default: $order_by = "ORDER BY name";
		}

		$sql = "SELECT *
				FROM `journal` ".$order_by;
		$sql_sec = "
				SELECT journal_id
				FROM `publication`
				GROUP BY journal_id
				HAVING COUNT(journal_id) > 0";
		$journal = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($journal)) foreach ($journal as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$journal[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_journal', 1);
		$this->smarty->assign('journal', $journal);
		$this->smarty->display('contents_journals.tpl');
	}

	/**
	 * Display edit form of the journal
	 *
	 * @param int $journal_id
	 */
	private function display_edit_journal($journal_id)
	{
		$sql = "SELECT *
				FROM `journal`
				WHERE id='$journal_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_journal', 1); //determine which part of template to use
		$this->smarty->assign('journal_id', $journal_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('issn', $data['issn']);
		$this->smarty->display('contents_journals.tpl');
	}

	/**
	 * Delete publisher without fk checks
	 *
	 * @param int $publisher_id
	 */
	private function delete_journal($journal_id)
	{
		$this->db->autoCommit(false);
		$res =& $this->db->query("DELETE FROM `journal` WHERE id='$journal_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		//we must perform SET NULL manually, the refference definition in database is disabled because MySQL refference definition in this case behave strange
		$res =& $this->db->query("UPDATE `publication` SET journal_id=0 WHERE journal_id='$journal_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}
	
	/**
	 * Display choose form of publications
	 *
	 * 
	 */
	private function display_publications($journal_id) {
		switch($this->get['order']) {
			default: $order_by = "ORDER BY title";
		}
		$sql = "SELECT id, title, submitter_id, pub_type
				FROM `publication`
				WHERE journal_id=$journal_id

				".$order_by; //select only complete conference2 with name and year, ommit 'private' non-complete records. Allow displaying non-complete records only to current user

		$publications = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);

		$this->smarty->assign('display_publications', 1);
		$this->smarty->assign('publications', $publications);
		$this->smarty->display('contents_journals.tpl');

	}

	public function display_overview($publication_id)
	{
		$data = get_all_publication_data($this->db, $publication_id);
		$date_arr = explode('-', $data['issue_date']);
		foreach ($data as $i => $v)
		{
			$this->smarty->assign($i, $v);
		}

		//attributes
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attrib_storage` JOIN `attributes`
				ON attrib_storage.attributes_id=attributes.id
				WHERE attrib_storage.publication_id = '$publication_id'";
		$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($attributes)) {
			throw new Exception($attributes->getMessage());
		}
		$s = '';
		if (is_array($attributes))
		foreach ($attributes as $arr)
		{
			$s .=
			'<tr>
				<td><input type="checkbox" name="propagated_attributes[]" value="'.$arr['id'].'" '.($arr['confirmed']?'checked="checked" disabled="disabled"':'').' /></td>
				<td><strong>'.$arr['name'].'</strong></td>'.'
				<td>'.htmlspecialchars($arr['description']).'</td>'.'
				<td>'.htmlspecialchars($this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$arr['id']."' AND publication_id='$publication_id'")).'</td>
			</tr>';
		}
		$this->smarty->assign('attributes', $s);

		$user_name = $this->db->getOne("SELECT name FROM `submitter` WHERE id=?", $data['submitter_id']);
		$user_surname = $this->db->getOne("SELECT surname FROM `submitter` WHERE id=?", $data['submitter_id']);
		if (PEAR::isError($user_name))
			throw new SAException($user_name->getMessage());
		if (PEAR::isError($user_surname))
			throw new SAException($user_surname->getMessage());
		$this->smarty->assign('user_name', $user_name);
		$this->smarty->assign('user_surname', $user_surname);

		//issue_date
		$issue_date_a = $this->db->getOne("SELECT issue_date FROM `publication` WHERE id=$publication_id");
		$issue_date_a=substr($issue_date_a,0, 7);
		$this->smarty->assign('issue_date_a', $issue_date_a);


		$publisher=$data['publisher_id'];
		$publisher_address = $this->db->getOne("SELECT address FROM `publisher` WHERE id=?", $data['publisher_id']);
		$this->smarty->assign('publisher_address', $publisher_address);


		$conference_year_id=$data['conference_year_id'];	
		$journal_id=$data['journal_id'];	

		$authors = get_authors($this->db, $publication_id);
		$categories = get_categories($this->db, $publication_id);
		$conference_year= get_conference_year_data($this->db, $conference_year_id);
		$conference2= get_conference2_data($this->db, $conference_year_id);
		$journal= get_journal_data($this->db, $journal_id);


		$this->smarty->assign('short_look', 1); //determine which part of template to use
		$this->smarty->assign('pid', $publication_id);
		$this->smarty->assign('title', $data['title']);
		$this->smarty->assign('booktitle', $data['booktitle']);
		$this->smarty->assign('pages', $data['pages']);
		$this->smarty->assign('pub_type', $data['pub_type']);
		$this->smarty->assign('isbn', $data['isbn']);

		$dp = new DocumentProvider($this->db, true);
		$files = $dp->get_file_links($publication_id);
		$this->smarty->assign('attached_files', $files);

		$this->smarty->assign('authors', $authors);
		$this->smarty->assign('categories', $categories);
		$this->smarty->assign('conference_year', $conference_year['conference_year']);
		$this->smarty->assign('conference2', $conference2['conference2']);
		$this->smarty->assign('journal', $journal['journal']);
		$this->smarty->assign('issn', $journal['issn']);

		$this->smarty->assign('tab', 8);

		$submitter_id=$this->auth_module->get_id();
		$publ_count = $this->db->getOne("SELECT COUNT(*) FROM `submitter_has_publication` WHERE publication_id=$publication_id AND submitter_id!=$submitter_id");
		$this->smarty->assign('publ_count', $publ_count);


		$this->smarty->display('contents_journals.tpl');

	}
	/**
	 * Delete files formerly attached to the publication from its storage location
	 *
	 * @param int $pid
	 */
	public function delete_files($publication_id)
	{
		$dirpath = "../storage/$publication_id/";
		$handle = @opendir($dirpath);
		if (!$handle)
			return;
		while ($file = readdir($handle))
		{
			if (is_file($path="../storage/$publication_id/".$file))
				unlink($path);
		}
		closedir($handle);
		$res = $this->db->query("DELETE FROM `documents` WHERE publication_id=?", $publication_id);
		if (PEAR::isError($res))
			throw new SAException($res->getMessage());
    	if (!rmdir($dirpath))
    		throw new SAException("Directory couldn't be deleted.");
	}

	/**
	 * Add document to the publication
	 *
	 * @param int  $publication_id
	 * @param string $title publication title
	 */
	public function add_file($publication_id)
	{

		$file =& $_FILES['file']['tmp_name'];
		$name =& $_FILES['file']['name'];
		$extension = substr($name, strrpos($name, ".") + 1);
		if ((!is_uploaded_file($file) && $file))
			throw new SAException("Not an uploaded file!");
		if ($file) {
			$convertor = new Convertor($this->db, $publication_id);
			if(!file_exists("../storage/$publication_id"))
				mkdir("../storage/$publication_id");
			$res = move_uploaded_file($file, "../storage/$publication_id/".$name);
			$convertor->add_file($name);
			if (!$res) {
				throw new SAException("Can't move uploaded file.");
			}
			$convertor->finish();
		}
	}

}
?>